#!/usr/bin/perl

#Calculates fragment lambda values for DNA reads. Make sure "readLengthLambda.R" is in PATH, or edit the code to give the full pathname below at line 64.
#Provide a 2 column table formatted: "length  count"
#Runs as an interactive helper script for readLengthLambda.R. You can run that script by itself if you'd like, just mind the required command line arguments.

use Getopt::Long;
GetOptions (
	'infile=s' => \$infile,
	'out=s' => \$out,
	'low=n' => \$low,
	'high=n' => \$high,
	'clip=f' => \$clip,
	'calcTable' => \$table,
	'help' => \$help);

unless ($infile and $out) {
	$help = 1;
}

if ($help) {
	die "\nExample Usage: perl lambdaCalc.pl --infile [in.lenTable.txt] --out [out prefix] [other options]
	
	This script runs interactively to eye-check lambda estimates. Requires \"readLengthLambda.R\" in PATH (or edit line 50 of this code).
	
	Options:
		--infile, input file: 2-column fragment length frequency table consisting of \"length count\". 
			See \"lenTables/\" with Kistler et al 2017 BioRxiv Supp Data S2 for examples.
		--out, prefix for output files. WILL OVERWRITE WITHOUT WARNING.
		--low [n], use with --high to specify a specific fragment range to calculate a fit, overriding the best fit.
		--high [n], ibid.
		--clip [f], if any length has a greater proportion of the reads than this value, change value to 0. Can be useful with artifactual spikes.
		--help: this menu.\n\n";
}

if ($low or $high or $clip) {
	$mode = "manual";
}
else {
	$mode = "auto";
}

$low = 0 unless ($low);
$high = 0 unless ($high);
$clip = 0 unless ($clip);

$lens = $infile;

while (1) {
	`cat $lens | readLengthLambda.R $out $low $high $clip`;
	`open $out.lengthStat.pdf `;
	print "Please take a look at the plot and make a choice (1,2,3,4,5 => Keep, Override, Abort, Flag, Re-automate)\n";
	$choice = <STDIN>;
	chomp $choice;
	if ($choice !~ /^[12345KkOoAaFfRr]$/) {
		print "You have 5 choices. Please select one of them with 1-5...\n";
		$choice = <STDIN>;
		chomp $choice;
		if ($choice !~ /^[12345]$/) {
			die "\nInvalid choice. Goodbye.\n\n";
		}
	}
	if ($choice =~ /^[1Kk]$/) {
		&keep;
	}
	if ($choice =~ /^[2Oo]$/) {
		&override;
	}
	if ($choice =~ /^[3Aa]$/) {
		&abort;
	}
	if ($choice =~ /^[4Ff]$/) {
		&flag;
	}
	if ($choice =~ /^[5Rr]$/) {
		$low = 0;
		$high = 0;
		$mode = "auto";
	}
}

sub abort {
	print "Cleaning up and exiting. Goodbye.\n";
	open OUT, ">$out.lambdaStat.txt";
	print OUT "k-2.5% NA
k-50% NA
k-97.5% NA
N-2.5% NA
N-50% NA
N-97.5% NA
min-k NA
max-k NA
min-N NA
max-N NA
start NA
end NA
clipped-top NA
low NA
high NA
#DATA UNSUITABLE FOR LAMBDA CALCULATION\n";
	close OUT;
	exit;
}

sub flag {
	print "Something is wrong with your data and you'd like to take another look? Flagging the stat file and keeping the pdf. Goodbye.\n\n";
	open OUT, ">$out.lambdaStat.txt";
	print OUT "k-2.5% NA
k-50% NA
k-97.5% NA
N-2.5% NA
N-50% NA
N-97.5% NA
min-k NA
max-k NA
min-N NA
max-N NA
start NA
end NA
clipped-top NA
low NA
high NA
#MANUALLY FLAGGED FOR EVALUATION\n";
	close OUT;
	exit;
}

sub keep {
	print "Finalizing out files and exiting. Goodbye.\n\n";
	open OUT, ">>$out.lambdaStat.txt";
	if ($mode =~ "auto") {
		print OUT "#CALCULATED USING AUTOMATED PARAMETERS\n";
	}
	else {
		print OUT "#MANUAL OVERRIDE: ";
		if ($low != 0 or $high != 0) {
			print OUT "$low to $high";
			if ($clip != 0) {
				print OUT ", ";
			}
		}
		if ($clip == 0) {
			print OUT "\n";
		}
		else {
			print OUT "CLIPPED $clip\n";
		}
	}
	close OUT;
	exit;
}

sub override {
	print "low,high,clip values please:\n\n";
	$choice = <STDIN>;
	chomp $choice;
	unless ($choice =~ /^\d+,\d+,[\d.]+$/) {
		print "Try again please. Enter low and high length values in the format \"low(integer),high(integer),clip(float). e.g. 50,80,0.15\"\n";
		$choice = <STDIN>;
		chomp $choice;
		die "Invalid choice, goodbye\n" unless ($choice =~ /^\d+,\d+$/);
	}
	($low, $high, $clip) = split /,/, $choice;
	$mode = "manual";
}
